/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.pdd;

// string manipulations
import org.apache.commons.lang.StringUtils;

/**
 * One puzzle convertable to UML.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Puzzle.java 1721 2011-07-27 23:58:18Z guard $
 */
public final class Puzzle {

    /**
     * The name of the ticket.
     * @see #Puzzle(String)
     */
    private String ticket;

    /**
     * The prefix of the puzzle.
     */
    private String prefix = "";

    /**
     * The body of the puzzle.
     * @see #setBody(String)
     */
    @SuppressWarnings("PMD.AvoidStringBufferField")
    private final StringBuffer body = new StringBuffer();

    /**
     * The estimate in minutes. Zero means that the estimate
     * was NOT provided and we don't know the real value of the
     * puzzle size (in minutes).
     * @see #setEstimate(Integer)
     */
    private Integer estimate = 0;

    /**
     * Lines interval where the puzzle was found.
     * @see #setLines(String)
     */
    private String lines;

    /**
     * Possible wwner of the puzzle.
     */
    public enum Owner {
        /**
         * Uplink of the ticket.
         */
        UPLINK("uplink"),
        /**
         * Who is currently working with it.
         */
        AUTHOR("author"),
        /**
         * Someone else, who we're waiting for.
         */
        SOMEONE("someone");
        /**
         * Title.
         */
        private final String title;
        /**
         * Private ctor.
         * @param text The title
         */
        private Owner(final String text) {
            this.title = text;
        }
        /**
         * Get title.
         * @return The title of the owner
         */
        @Override
        public String toString() {
            return this.title;
        }
    };

    /**
     * Owner of the puzzle.
     * @see #setOwner(Puzzle.Owner)
     */
    private Puzzle.Owner owner = Puzzle.Owner.SOMEONE;

    /**
     * Public ctor.
     * @param tkt The ticket number
     */
    public Puzzle(final String tkt) {
        this.ticket = tkt;
    }

    /**
     * Set prefix used in the source file.
     * @param pfx The prefix
     */
    public void setPrefix(final String pfx) {
        this.prefix = pfx;
    }

    /**
     * Append string to the body.
     * @param appendix What to append to the body
     */
    public void appendBody(final String appendix) {
        this.body.append(StringUtils.strip(appendix)).append(" ");
    }

    /**
     * Set estimate of the puzzle, in minutes.
     * @param min Minutes
     */
    public void setEstimate(final Integer min) {
        this.estimate = min;
    }

    /**
     * Set lines interval.
     * @param first The first line of the puzzle
     * @param last The last line of the puzzle
     */
    public void setLines(final Integer first, final Integer last) {
        final StringBuilder builder = new StringBuilder();
        this.lines = builder.append(first).append("-").append(last).toString();
    }

    /**
     * Set owner of the puzzle.
     * @param owr The owner
     */
    public void setOwner(final Puzzle.Owner owr) {
        this.owner = owr;
    }

    /**
     * Get owner of the puzzle.
     * @return The owner
     */
    public Puzzle.Owner getOwner() {
        return this.owner;
    }

    /**
     * Get ticket name.
     * @return The name of the ticket
     */
    public String getTicket() {
        return this.ticket;
    }

    /**
     * Get prefix.
     * @return The prefix
     */
    public String getPrefix() {
        return this.prefix;
    }

    /**
     * Get body of the puzzle.
     * @return The body
     */
    public String getBody() {
        final String txt = this.body.toString();
        if (txt.length() == 0) {
            return "empty";
        }
        return StringUtils.strip(txt);
    }

    /**
     * Get estimate in minutes.
     * @return Minutes
     */
    public Integer getEstimate() {
        return this.estimate;
    }

    /**
     * Get lines interval where the puzzle was found.
     * @return The lines interval, like "15-18"
     */
    public String getLines() {
        return this.lines;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("[#").append(this.ticket).append(",");
        if (this.estimate != 0) {
            buf.append(this.estimate).append("min");
        } else {
            buf.append("no-estimate");
        }
        buf.append("]");
        return buf.toString();
    }

}
